#
# $Header:$
#

use 5.008;
use ExtUtils::MakeMaker;
use File::Spec;
use Config;
use Cwd;

# minimum required version of Glib with which we can work.
my $req_glib_ver = '1.00rc1';
my $req_gtk2_ver = '1.00rc1';

# Writing a fake Makefile ensures that CPAN will pick up the correct
# dependencies and install them.
unless (eval "use ExtUtils::Depends;"
           . "use Glib::PkgConfig;"
           # just seeing if Glib is available isn't enough, make sure
           # it's recent enough, too 
           . "use Glib '$req_glib_ver';"
           . "use Gtk2 '$req_gtk2_ver';"
           . "use Gtk2::CodeGen;"
           . "1") {
   warn "$@\n";
   WriteMakefile(
         PREREQ_FATAL => 1,
         PREREQ_PM    => {
            Glib::              => $req_glib_ver, # keep this in sync
            Gtk2::              => $req_gtk2_ver, # keep this in sync
            ExtUtils::Depends:: => 0.1,
         },
   );
   exit 1; # not reached
}

my %pkgcfg = Glib::PkgConfig->find ('gtk+-2.0');

mkdir 'build', 0777;

#
# autogeneration
#
# we have a whole lot of types which need cast macros, typemap entries,
# and package mapping registration; instead of writing this by hand, we
# use a map file which defines how things are laid out, and then use
# some boilerplate code to convert that map into a header, typemap, and
# xsh file.  see the docs for Gtk2::CodeGen for more info.
#
# we'll use all the default filenames.
#
Gtk2::CodeGen->parse_maps ('gtk2perl_deprecated');

# one more command generates code to boot all the various extra XS modules.
# we need this because we have lots of XS files without corresponding PMs to
# bootstrap them.
Gtk2::CodeGen->write_boot (ignore => 'Deprecated');

#
# since we've started down the dark path of breaking from standard 
# MakeMaker, forever will it dominate our destiny; we must specify
# install paths for the modules and man files by hand.
#
my %pm_files = (
	'Deprecated.pm' => '$(INST_LIBDIR)/Deprecated.pm',
);
my %pod_files = (
	'Deprecated.pm' => '$(INST_MAN3DIR)/Deprecated.$(MAN3EXT)',
);

# now we're ready to start creating the makefile.
# we need to use ExtUtils::Depends to get relevant information out of
# the Glib extension, and to save config information for other modules which
# will chain from this one.

my $gtk2dep = ExtUtils::Depends->new ('Gtk2::Deprecated', 'Gtk2', 'Glib');
# Glib added -I. for us, but we'll need to add -I./build so we can get to
# the autogenerated files.  we do this kindof as a public service to client
# modules, as these will carry through via the depends mechanism.
$gtk2dep->set_inc ($pkgcfg{cflags} . ' -I./build ');
$gtk2dep->set_libs ($pkgcfg{libs} . find_extra_libs());
$gtk2dep->add_pm (%pm_files);
$gtk2dep->add_xs (<xs/*.xs>);
my $cwd = cwd();
$gtk2dep->add_typemaps (map {File::Spec->catfile($cwd,$_)}
		     'build/gtk2perl_deprecated.typemap',
		     );
$gtk2dep->add_headers ('gtk2perl_deprecated.h');

$gtk2dep->install (qw(gtk2perl_deprecated.h build/gtk2perl_deprecated-autogen.h));
$gtk2dep->save_config ('build/IFiles.pm');

WriteMakefile(
    NAME          => 'Gtk2::Deprecated',
    VERSION_FROM  => 'Deprecated.pm', # finds $VERSION
    ABSTRACT_FROM => 'Deprecated.pm', # retrieve abstract from module
    XSPROTOARG    => '-noprototypes',
    MAN3PODS      => \%pod_files,
    # PREREQ see beginning of file
    PREREQ_FATAL => 1,
    PREREQ_PM    => {
        Glib::  => $req_glib_ver, # keep this in sync
        Gtk2::  => $req_gtk2_ver, # keep this in sync
    },
    $gtk2dep->get_makefile_vars,
);

#
# there's a bug in the MakeMaker output which results in .xs files in a
# subdirectory being turned into .o files in this dir.  the bug is a missing
# -o on the compiler command line.  this is a hackish fix for it.
#
package MY;
use Cwd;

sub const_cccmd {
	my $inherited = shift->SUPER::const_cccmd (@_);
	$inherited .= ' -o $@';
	$inherited;
}

sub postamble
{
	my @dirs = qw{rpms rpms/BUILD rpms/RPMS rpms/SOURCES
		      rpms/SPECS rpms/SRPMS};
	my $cwd = getcwd();

	"
realclean ::
	-rm -Rf build

# the tmp-xxx stuff is just so that only the pl files get installed
install-\%: %
	\@\$(MKPATH) tmp-\$+/
	\@\$(CP) \$+/*.* tmp-\$+/
	\@\$(MOD_INSTALL) ./tmp-\$+/ \\
		\$(PREFIX)/share/doc/perl-\$(DISTNAME)/\$+
	\@\$(RM_RF) tmp-\$+/

rpms/:
	-mkdir @dirs

perl-\$(DISTNAME).spec: perl-\$(DISTNAME).spec.in
	sed -e \"s/\@VERSION\@/\$(VERSION)/g\" -e \"s/\@SOURCE\@/\$(DISTNAME)-\$(VERSION).tar.gz/g\" \$< > \$@

dist-rpms: Makefile dist perl-\$(DISTNAME).spec rpms/
	cp \$(DISTNAME)-\$(VERSION).tar.gz rpms/SOURCES/
	rpmbuild -ba --define \"_topdir $cwd/rpms\" perl-\$(DISTNAME).spec
"
}
package main;

# this probably needs to go into ExtUtils::Depends.
sub find_extra_libs {
	# right now we need this terrible hack only for windows.
	return "" unless $^O eq "MSWin32";

	# win32 does not allow unresolved symbols in libraries, but
	# Gtk2 depends on symbols in the dll created for Glib.
	# so, we have to break all this nice abstraction and encapsulation
	# and find the actual Glib.dll and Glib.lib installed by perl when
	# the Glib module was built, and add it to the list of lib files.
	#
	# say it with me: "i hate win32."

	my $retstring = "";

	use File::Find;
	find (sub {
			$retstring .= " ".$File::Find::name
				if /Glib.lib/i;
		}, @INC);

	return $retstring;
}
